/*
 * Copyright (C) 2021. Huawei Technologies Co., Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 and
 * only version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 */
#include "nvme_internal.h"

bool spdk_nvme_ns_pi_md_start(struct spdk_nvme_ns *ns)
{
	struct spdk_nvme_ns_data *nsdata = spdk_nvme_ns_get_data(ns);
	return nsdata->dps.md_start == 1;
}

bool spdk_nvme_ns_is_dataset_mng_supported(struct spdk_nvme_ns *ns)
{
	return (ns->flags & SPDK_NVME_NS_DEALLOCATE_SUPPORTED) == 1;
}

int nvme_ns_get_common_data(struct spdk_nvme_ctrlr *ctrlr, struct spdk_nvme_ns_data *nsdata)
{
	if (ctrlr == NULL || nsdata == NULL) {
		SPDK_ERRLOG("Parameter error\n");
		return -1;
	}

	struct nvme_completion_poll_status  status;
	int rc = 0;
	status.done = false;

	if (spdk_nvme_ctrlr_is_ns_manage_supported(ctrlr)) {
		rc = nvme_ctrlr_cmd_identify(ctrlr, SPDK_NVME_IDENTIFY_NS, 0, SPDK_NVME_GLOBAL_NS_TAG, 0,
					     nsdata, sizeof(*nsdata), nvme_completion_poll_cb, &status);
	} else {
		rc = nvme_ctrlr_cmd_identify(ctrlr, SPDK_NVME_IDENTIFY_NS, 0, 1, 0,
					     nsdata, sizeof(*nsdata), nvme_completion_poll_cb, &status);
	}
	if (rc != 0) {
		return rc;
	}

	if (nvme_wait_for_completion_robust_lock(ctrlr->adminq, &status, &ctrlr->ctrlr_lock)) {
		SPDK_ERRLOG("Failed to identify nsdata, sct[%x], sc[%x]\n", status.cpl.status.sct,
			    status.cpl.status.sc);
		return -1;
	}

	return 0;
}

bool spdk_nvme_ns_is_allocated(struct spdk_nvme_ctrlr *ctrlr, uint16_t nsid)
{
	struct spdk_nvme_ns_data nsdata;
	struct nvme_completion_poll_status  status;
	int rc = 0;

	if (ctrlr == NULL || nsid == 0) {
		SPDK_ERRLOG("Parameter error. ns[%u]\n", nsid);
		return false;
	}

	if (!spdk_nvme_ctrlr_is_ns_manage_supported(ctrlr)) {
		return true;
	}

	nsdata.ncap = 0;
	status.done = false;
	rc = nvme_ctrlr_cmd_identify(ctrlr, SPDK_NVME_IDENTIFY_NS_ALLOCATED, 0, nsid, 0,
				     &nsdata, sizeof(nsdata), nvme_completion_poll_cb, &status);
	if (rc != 0) {
		SPDK_ERRLOG("Failed to identify ns[%u]'s nsdata. rc[%d]\n", nsid, rc);
		return false;
	}

	if (nvme_wait_for_completion_robust_lock(ctrlr->adminq, &status, &ctrlr->ctrlr_lock)) {
		SPDK_ERRLOG("Failed to identify ns[%u]'s nsdata, sct[%x], sc[%x]\n",
			    nsid, status.cpl.status.sct, status.cpl.status.sc);
		return false;
	}

	return (nsdata.ncap != 0);
}

bool
spdk_nvme_ns_ctrl_is_failed(struct spdk_nvme_ns *ns)
{
	return ns->ctrlr->is_failed;
}
